/*
 * Autogenerated by the Meson build system.
 * Do not edit, your changes will be lost.
 */

#pragma once

#define NICE_VERSION_MAJOR 0

#define NICE_VERSION_MICRO 19

#define NICE_VERSION_MINOR 1

#define NICE_VERSION_NANO 1

